----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:40:50 02/18/2010 
-- Design Name: 
-- Module Name:    DDRIOBUF - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

-----------------------------------------------------------
--
-- SRAMo̓obt@
-- |[g}bv錾
--
-----------------------------------------------------------
entity SRAM_IF is
	port (
		-- Oo̓|[g
		oAddr:		out		std_logic_vector(15 downto 0);
		oData:		inout	std_logic_vector(15 downto 0);
		oOEn:		out		std_logic;
		oWEn:		out		std_logic;
		
		-- o̓|[g
		iAddr:		in		std_logic_vector(15 downto 0);
		iWrite:		in		std_logic_vector(15 downto 0);
		iRead:		out		std_logic_vector(15 downto 0);
		iOEn:		in		std_logic;
		iWEn:		in		std_logic;
		
		-- W[NbN(80MHz)
		-- W[NbN(80MHz/270deg)
		Clock0:		in		std_logic;
		Clock180:	in		std_logic
	);
end SRAM_IF;

architecture Behavioral of SRAM_IF is
	signal	Data	:	std_logic_vector(15 downto 0);
	signal	Buff	:	std_logic_vector(15 downto 0);
	signal	State	:	std_logic_vector(15 downto 0);
	signal	Latch	:	std_logic;
	signal	InputEn	:	std_logic;
begin

------------------------------------------------------------
-- OutputEnable and WriteEnable
	
	process ( Clock0, Clock180 ) begin
	
	-- NbN0̗オOEnۑ܂
	if ( Clock180'event and Clock180='1' ) then
		Latch <= not iOEn;
		InputEn <= Latch;
	end if;
	
	-- NbN270̗オŃAhXb`܂
	if ( Clock180'event and Clock180='1' ) then
		oAddr <= iAddr;
	end if;
	end process;

------------------------------------------------------------
-- OutputEnable and WriteEnable

	-- OutputEnableAo
	OEn_ODDR2 : ODDR2
		generic map(
			DDR_ALIGNMENT => "NONE",-- Sets output alignment to "NONE", "C0", "C1" 
			INIT	=> '1',			-- Sets initial state of the Q output to '0' or '1'
			SRTYPE	=> "SYNC"		-- Specifies "SYNC" or "ASYNC" set/reset
		)
		port map (
			Q	=> oOEn,	-- 1-bit output data
			C0	=> Clock0,	-- 1-bit clock input
			C1	=> Clock180,-- 1-bit clock input
			CE	=> '1',		-- 1-bit clock enable input
			D0	=> iOEn,	-- 1-bit data input (associated with C0)
			D1	=> '1',		-- 1-bit data input (associated with C1)
			R	=> '0',		-- 1-bit reset input
			S	=> '0'		-- 1-bit set input
		);
	
	-- WriteEnableAo
	WEn_ODDR2 : ODDR2
		generic map(
			DDR_ALIGNMENT => "NONE",-- Sets output alignment to "NONE", "C0", "C1" 
			INIT	=> '1',			-- Sets initial state of the Q output to '0' or '1'
			SRTYPE	=> "SYNC"		-- Specifies "SYNC" or "ASYNC" set/reset
		)
		port map (
			Q	=> oWEn,	-- 1-bit output data
			C0	=> Clock0,	-- 1-bit clock input
			C1	=> Clock180,-- 1-bit clock input
			CE	=> '1',		-- 1-bit clock enable input
			D0	=> iWEn,	-- 1-bit data input (associated with C0)
			D1	=> '1',		-- 1-bit data input (associated with C1)
			R	=> '0',		-- 1-bit reset input
			S	=> '0'		-- 1-bit set input
		);
	
	
------------------------------------------------------------
-- IOBUF/DataBus/TriState generate loop

	SRAM_IO_GEN : for index in 0 to 15 generate
	
		-- o̓obt@
		IOBUF_inst : IOBUF
			generic map (
--				DRIVE => 12,
				IBUF_DELAY_VALUE => "0", -- Specify the amount of added input delay for buffer, "0"-"16" (Spartan-3E/3A only)
				IFD_DELAY_VALUE => "4", -- Specify the amount of added delay for input register, "AUTO", "0"-"8" (Spartan-3E/3A only)
				IOSTANDARD => "DEFAULT"
--				SLEW => "SLOW"
				)
			port map (
				O	=> Buff(index),		-- Buffer output
				IO	=> oData(index),	-- Buffer inout port (connect directly to top-level port)
				I	=> Data(index),		-- Buffer input
				T	=> State(index)		-- 3-state enable input 
			);
			
		-- RXe[gobt@
		TBUF_ODDR2 : ODDR2
			generic map(
				DDR_ALIGNMENT => "NONE",-- Sets output alignment to "NONE", "C0", "C1" 
				INIT	=> '1',			-- Sets initial state of the Q output to '0' or '1'
				SRTYPE	=> "SYNC"		-- Specifies "SYNC" or "ASYNC" set/reset
			)
			port map (
				Q	=> State(index),-- 1-bit output data
				C0	=> Clock0,		-- 1-bit clock input
				C1	=> Clock180,	-- 1-bit clock input
				CE	=> '1',			-- 1-bit clock enable input
				D0	=> iWEn,			-- 1-bit data input (associated with C0)
				D1	=> '1',			-- 1-bit data input (associated with C1)
				R	=> '0',			-- 1-bit reset input
				S	=> '0'			-- 1-bit set input
			);
			
		-- o̓obt@
		OBUF_ODDR2 : ODDR2
			generic map(
				DDR_ALIGNMENT => "NONE",-- Sets output alignment to "NONE", "C0", "C1" 
				INIT	=> '0',			-- Sets initial state of the Q output to '0' or '1'
				SRTYPE	=> "SYNC"		-- Specifies "SYNC" or "ASYNC" set/reset
			)
			port map (
				Q	=> Data(index),		-- 1-bit output data
				C0	=> Clock0,			-- 1-bit clock input
				C1	=> Clock180,		-- 1-bit clock input
				CE	=> '1',				-- 1-bit clock enable input
				D0	=> iWrite(index),	-- 1-bit data input (associated with C0)
				D1	=> iWrite(index),	-- 1-bit data input (associated with C1)
				R	=> '0',				-- 1-bit reset input
				S	=> '0'				-- 1-bit set input
			);
			
		-- ̓obt@
		FDRSE_inst : FDRSE_1
			generic map (
				INIT => '0' -- Initial value of register ('0' or '1')  
			)
			port map (
				Q	=> iRead(index),-- Data output
				C	=> Clock180,	-- Clock input
				CE	=> InputEn,		-- Clock enable input
				D	=> Buff(index), -- Data input
				R	=> '0',			-- Synchronous reset input
				S	=> '0'			-- Synchronous set input
			);
	end generate;
	
end Behavioral;

