library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;

-----------------------------------------------------------
--
-- fBXvCƃTvÕf[^oX؂ւ郍WbN
-- |[g}bv錾
--
-----------------------------------------------------------
entity dso_page_switch is
	port (
		-- OSRAMoX
		ExAddrA:	out		std_logic_vector(15 downto 0);
		ExAddrB:	out		std_logic_vector(15 downto 0);
		ExDataA:	inout	std_logic_vector(15 downto 0);
		ExDataB:	inout	std_logic_vector(15 downto 0);
		ExOEnA:		out		std_logic;
		ExOEnB:		out		std_logic;
		ExWEnA:		out		std_logic;
		ExWEnB:		out		std_logic;
		
		-- SRAMoX
		InAddrA:	in		std_logic_vector(15 downto 0);
		InAddrB:	in		std_logic_vector(15 downto 0);
		InWriteA:	in		std_logic_vector(15 downto 0);
		InWriteB:	in		std_logic_vector(15 downto 0);
		InReadA:	out		std_logic_vector(15 downto 0);
		InReadB:	out		std_logic_vector(15 downto 0);
		InOEnA:		in		std_logic;
		InOEnB:		in		std_logic;
		InWEnA:		in		std_logic;
		InWEnB:		in		std_logic;
		
		-- oXXCb`M
		-- W[NbN(80MHz)
		-- W[NbN(80MHz/270deg)
		Bank:		in		std_logic;
		Clock80:	in		std_logic;
		Clock180:	in		std_logic
	);
end dso_page_switch;
-----------------------------------------------------------


-----------------------------------------------------------
--
-- WbN
--
-----------------------------------------------------------
architecture Behavioral of dso_page_switch is
	
	-- SRAMC^[tFCXR|[lg
	component SRAM_IF
	port (
		-- Oo̓|[g
		oAddr:		out		std_logic_vector(15 downto 0);
		oData:		inout	std_logic_vector(15 downto 0);
		oOEn:		out		std_logic;
		oWEn:		out		std_logic;
		
		-- o̓|[g
		iAddr:		in		std_logic_vector(15 downto 0);
		iWrite:		in		std_logic_vector(15 downto 0);
		iRead:		out		std_logic_vector(15 downto 0);
		iOEn:		in		std_logic;
		iWEn:		in		std_logic;
		Clock0:		in		std_logic;
		Clock180:	in		std_logic
	);
	end component;

	signal	mReadA, mReadB, mWriteA, mWriteB
		:	std_logic_vector(15 downto 0);
	signal	mAddrA, mAddrB
		:	std_logic_vector(15 downto 0);
	signal	mWEnA, mWEnB, mOEnA, mOEnB
		:	std_logic;
	
begin
------------------------------------------------------------
-- f[^oXXCb`
	process ( Bank, InAddrA, InAddrB, InWriteA, InWriteB ) begin
		
		-- Bank0̂Ƃ́AXg[gڑ
		if ( Bank='0' ) then
			mAddrA <= InAddrA;
			mAddrB <= InAddrB;
			mWriteA <= InWriteA;
			mWriteB <= InWriteB;
			
		-- Bank1̎́ANXڑ
		else
			mAddrA <= InAddrB;
			mAddrB <= InAddrA;
			mWriteA <= InWriteB;
			mWriteB <= InWriteA;
		end if;
	end process;
	
	process ( Bank, InOEnA, InOEnB, InWEnA, InWEnB ) begin
		
		-- Bank0̂Ƃ́AXg[gڑ
		if ( Bank='0' ) then
			mOEnA <= InOEnA;
			mOEnB <= InOEnB;
			mWEnA <= InWEnA;
			mWEnB <= InWEnB;
			
		-- Bank1̎́ANXڑ
		else
			mOEnA <= InOEnB;
			mOEnB <= InOEnA;
			mWEnA <= InWEnB;
			mWEnB <= InWEnA;
		end if;
	end process;
	
	process ( Bank, mReadA, mReadB ) begin
		
		-- Bank0̂Ƃ́AXg[gڑ
		if ( Bank='0' ) then
			InReadA <= mReadA;
			InReadB <= mReadB;
			
		-- Bank1̎́ANXڑ
		else
			InReadA <= mReadB;
			InReadB <= mReadA;
		end if;
	end process;

------------------------------------------------------------
-- 񓯊SRAMC^[tFCX

	SRAM_instA : SRAM_IF
		port map(
			-- Oo̓|[g
			oAddr	=> ExAddrA,
			oData	=> ExDataA,
			oOEn	=> ExOEnA,
			oWEn	=> ExWEnA,
			
			-- o̓|[g
			iAddr	=> mAddrA,
			iWrite	=> mWriteA,
			iRead	=> mReadA,
			iOEn	=> mOEnA,
			iWEn	=> mWEnA,
			Clock0	=> Clock80,
			Clock180=> Clock180
		);
	
	SRAM_instB : SRAM_IF
		port map(
			-- Oo̓|[g
			oAddr	=> ExAddrB,
			oData	=> ExDataB,
			oOEn	=> ExOEnB,
			oWEn	=> ExWEnB,
			
			-- o̓|[g
			iAddr	=> mAddrB,
			iWrite	=> mWriteB,
			iRead	=> mReadB,
			iOEn	=> mOEnB,
			iWEn	=> mWEnB,
			Clock0	=> Clock80,
			Clock180=> Clock180
		);
end Behavioral;
